/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};
gc.debugVariable = false;

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener("gc-databind-ready", function() {
  /*
   *   Add custom computed value databindings here, using the following method:
   *
   *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
   *
   *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
   *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
   *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
   */

  // For example, a simple computed values based on simple expression
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

  // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
  // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

  // Event 1 to n bindings
  /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
    */
  gc.databind.registry.bind(
    "widget.ti_tpslp_main.connectionStatus",
    "widget.my_transport.status"
  );
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener("gc-nav-ready", function() {
  /*
   *   Add custom actions for menu items using the following api:
   *
   *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
   *
   *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
   *   param runable - function that performs the custom action.
   *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
   *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
   */

  // For example,
  // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

  // Alternatively, to programmatically disable a menu action at any time use:
  // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

  // bc: [...
  // Code that uses templateObj.$ to reference components hosted by ti-tpslp-main will not work,
  // since the vtabcontainer that is referenced by templateObj.$.ti_widget_vtabcontainer is not
  // the same as the one that is being used by the application.

  gc.nav.registerAction(
    "show_introduction",
    function() {
      // if ((templateObj) && (templateObj.$)) {
      //   templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
      //   templateObj.$.ti_widget_menuitem.hideFullMenu();
      // }
      var vmenu = document.querySelector("ti-widget-vtabcontainer");
      if (vmenu) {
        vmenu.selectedIndex = 0;
        vmenu.hideFullMenu();
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );
  gc.nav.registerAction(
    "show_registers",
    function() {
      // if ((templateObj) && (templateObj.$)) {
      //   templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
      //   templateObj.$.ti_widget_menuitem.hideFullMenu();
      // }
      var vmenu = document.querySelector("ti-widget-vtabcontainer");
      if (vmenu) {
        vmenu.selectedIndex = 0;
        vmenu.hideFullMenu();
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );
  gc.nav.registerAction(
    "show_motor_control",
    function() {
      // if ((templateObj) && (templateObj.$)) {
      //   templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
      //   templateObj.$.ti_widget_menuitem.hideFullMenu();
      // }
      var vmenu = document.querySelector("ti-widget-vtabcontainer");
      if (vmenu) {
        vmenu.selectedIndex = 2;
        vmenu.hideFullMenu();
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  gc.nav.registerAction(
    "load_registers",
    function() {
      if (templateObj && templateObj.$) {
        gc.File.browseAndLoad(
          null,
          {
            bin: false
          },
          function(contents, properties) {
            var regData = null;
            try {
              regData = JSON.parse(contents);
              if (regData) {
                templateObj.setRegisterData(regData);
                templateObj.$.ti_widget_eventlog_view.log(
                  "info",
                  "Load Registers: loaded from :" + properties.localPath
                );
              }
            } catch (ex) {
              templateObj.logMsgAndNotify(
                "error",
                "Load Registers: Could not parse file:" + ex,
                false,
                true
              );
            }
          }
        );
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  var saveText = function(fname, text, mime) {
    var blob = new Blob([text], {
      type: mime
    });
    saveAs(blob, fname);
    return false;
  };
  gc.nav.registerAction(
    "save_registers",
    function() {
      if (templateObj && templateObj.$) {
        var regData = templateObj.getRegisterData();
        var regDataJson = JSON.stringify(regData);
        if (gc.desktop.isDesktop()) {
          gc.File.browseAndSave(regDataJson, "regs.json", "*.json", function(
            result,
            errorInfo
          ) {
            if (errorInfo) {
              templateObj.logMsgAndNotify(
                "error",
                "Save Registers: " + errorInfo,
                false,
                true
              );
            } else if (result && result.localPath) {
              templateObj.$.ti_widget_eventlog_view.log(
                "info",
                "Save Registers: saved to " + result.localPath
              );
            }
          });
        } else {
          saveText("regs.json", regDataJson, "text/plain;charset=utf-8");
          templateObj.$.ti_widget_eventlog_view.log(
            "info",
            "Save Registers: regs.json file downloaded"
          );
        }
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  gc.nav.registerAction(
    "save_registers_as",
    function() {
      if (templateObj && templateObj.$) {
        var regData = templateObj.getRegisterData();
        var regDataJson = JSON.stringify(regData);
        if (gc.desktop.isDesktop()) {
          gc.File.browseAndSave(regDataJson, "regs.json", "*.json", function(
            result,
            errorInfo
          ) {
            if (errorInfo) {
              templateObj.logMsgAndNotify(
                "error",
                "Save Registers As: " + errorInfo,
                false,
                true
              );
            } else if (result && result.localPath) {
              templateObj.$.ti_widget_eventlog_view.log(
                "info",
                "Save Registers As: saved to " + result.localPath
              );
            }
          });
        } else {
          saveText("regs.json", regDataJson, "text/plain;charset=utf-8");
          templateObj.$.ti_widget_eventlog_view.log(
            "info",
            "Save Registers: regs.json file downloaded"
          );
        }
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  gc.nav.registerAction(
    "open_log_pane",
    function() {
      if (templateObj && templateObj.$) {
        templateObj.$.ti_widget_eventlog_view.openView();
      }
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  gc.nav.registerAction("open_device_settings_dialog", function() {
    if (!templateObj) {
      templateObj = document.querySelector("#template_obj");
    }
    templateObj.$.ti_tpslp_main.openDeviceSettingsDialog();
  });

  gc.nav.registerAction(
    "open_scripting_dialog",
    function() {
      window.open("app/scripting.html", "_evm_scripting");
    },
    function() {
      return true;
    },
    function() {
      return true;
    }
  );

  gc.nav.registerAction("save_config", function() {
    if (!templateObj) {
      templateObj = document.querySelector("#template_obj");
    }
    templateObj.$.ti_tpslp_main.saveConfiguration();
  });

  gc.nav.registerAction("upload_config", function() {
    if (!templateObj) {
      templateObj = document.querySelector("#template_obj");
    }
    templateObj.$.ti_tpslp_main.sendCommand("load");
  });
});

/**
 * register changed listener
 * @param regName:  specified by the bind function used when the listener was created (see below)
 * @param newValue: the value the register was changed to
 * @param oldValue: the value the register was before it changed
 * @param progress: any progress callback associated with this listener (ignore)
 */
var _registerChangedListener = function(regName, oldValue, newValue, progress) {
  var maxBitWidth = 16; // or whatever the bitwidth is for your target device
  var hexValueStr = gc.utils.getHexString(newValue, maxBitWidth, "0x");
  console.log("Register " + regName + " changed: newValue = " + hexValueStr);
  var regPage = document.querySelector("ti-widget-register-page");
  if (regPage) {
    //regPage.refreshGrid(0,false,true);  // collapse all of the register groups
    regPage.refreshGrid();
  }
};

/*
 *  Boilerplate code for working with components in the application gist
 */

var initComplete = false;
var templateObj;
var mainComponentObj;
var aboutBox;
var registerModel;
var originalProductName = "";

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
  templateObj = document.querySelector("#template_obj");
  mainComponentObj = document.querySelector("#ti_tpslp_main");

  // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
  // before trying to access components in the application.
  mainComponentObj.addEventListener("dom-change", function() {
    if (initComplete) return;
    this.async(function() {
      mainComponentObj = document.querySelector("#ti_tpslp_main");
      aboutBox = document.querySelector("ti-widget-aboutbox");
      aboutBox.addEventListener("aboutbox_opening", function() {
        aboutBox.applicationVersion = "2.0.0";
        aboutBox.softwareManifestLink =
          "docs/Scalable PMICs GUI_2.0_manifest.html";
        aboutBox.licenseLink = "docs/SRAS Prod_License_Click 2019.pdf";
      });
      initComplete = true;
      console.log("Application template has been stamped.");
      mainComponentObj.$.ti_widget_toast.hideToast();
      templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
      registerModel = Polymer.dom(document).querySelector("ti-model-register");
      if (registerModel) {
        registerModel.addEventListener("usbHidModelReceivingData", function(
          event
        ) {
          templateObj.logMsgAndNotify(
            "debug",
            "Receiving data from target.",
            false,
            false
          );
        });
      } else {
        console.log(
          "no ti-model-register component found in this application!"
        );
      }

      // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
      // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

      // open the eventlog viewer when the user clicks on the book icon in the status bar:
      //bc: The following is obsolete (appiconclicked event no longer fired by statusbar)
      //   templateObj.$.ti_widget_eventlog_view
      //     .closeView(); // ensure that view states are in sync with application
      //   templateObj.$.ti_widget_statusbar.addEventListener(
      //     "appiconclicked",
      //     function (event) {
      //       templateObj.$.ti_widget_eventlog_view.toggleView();
      //     });

      // Expand vtabcontainer nav bar when user clicks on menu icon or 'Menu' label
      templateObj.toggleMenu = function(event) {
        // templateObj.$.ti_widget_vtabcontainer.isExpanded = !
        //   templateObj.$.ti_widget_vtabcontainer.isExpanded;
        var vmenu = document.querySelector("ti-widget-vtabcontainer");
        if (vmenu) {
          vmenu.isExpanded = !vmenu.isExpanded;
        }
      };
      // templateObj.$.ti_widget_icon_button_menu.addEventListener(
      //   'click', templateObj.toggleMenu);
      // templateObj.$.ti_widget_label_menu.addEventListener('click',
      //   templateObj.toggleMenu);

      if (mainComponentObj.$.ti_widget_register_page) {
        // called when the user clicks the Write Register button:
        mainComponentObj.writeReg = function(event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(
            detailsObj.statusType,
            detailsObj.statusMsg,
            false,
            true
          );
        };
        // called when the user clicks the Write All Registers button:
        mainComponentObj.writeRegs = function(event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(
            detailsObj.statusType,
            detailsObj.statusMsg,
            false,
            true
          );
        };

        // called when the user clicks the Read Register button:
        mainComponentObj.readReg = function(event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(
            detailsObj.statusType,
            detailsObj.statusMsg,
            false,
            true
          );
        };

        // called when the user clicks the Read All Registers button:
        mainComponentObj.readRegs = function(event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(
            detailsObj.statusType,
            detailsObj.statusMsg,
            false,
            true
          );
        };

        // called when the user changes the immediate / deferred mode
        mainComponentObj.updateModeChanged = function(event) {
          var detailsObj = event.detail;
          templateObj.$.ti_widget_eventlog_view.log(
            "info",
            "request to change update mode to " + detailsObj.mode.label
          );
        };

        // called when the user changes the auto read refresh rate
        mainComponentObj.autoReadConfigChanged = function(event) {
          var detailsObj = event.detail;
          templateObj.$.ti_widget_eventlog_view.log(
            "info",
            "request to change automatic read interval to " +
              detailsObj.autoRead.label
          );
        };

        if (mainComponentObj.$.ti_widget_register_page) {
          // demo of handling when the user clicks the Write Register button
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "write_reg",
            mainComponentObj.writeReg
          );
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "write_regs",
            mainComponentObj.writeRegs
          );
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "read_reg",
            mainComponentObj.readReg
          );
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "read_regs",
            mainComponentObj.readRegs
          );
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "update_mode_changed",
            mainComponentObj.updateModeChanged
          );
          mainComponentObj.$.ti_widget_register_page.addEventListener(
            "auto_read_config_changed",
            mainComponentObj.autoReadConfigChanged
          );
        }
      }

      var usbHidService = gc.services["ti-service-usbhid"];
      if (usbHidService) {
        usbHidService.addEventListener("usbHidError", function(event) {
          var detailsObj = event.detail;
          templateObj.logMsgAndNotify(
            "error",
            detailsObj.info + ": " + detailsObj.error,
            false,
            true
          );
        });
      }

      templateObj.getRegisterData = function() {
        var regData = {
          signature: "oneui-register-data",
          data: []
        };
        // var regBlocks = templateObj.$.ti_widget_register_page
        //   .regBlocks;
        var regPage = document.querySelector("ti-widget-register-page");
        var regBlocks = regPage.regBlocks;
        for (var i = 0; i < regBlocks.length; i++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[i].registers.length; r++) {
            var reg = regBlocks[i].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            var item = {
              idx: r,
              id: regName,
              value: reg.value
            };
            registersData.push(item);
          }
          regData.data.push(registersData);
        }
        var strJson = JSON.stringify(regData);
        return strJson;
      };

      templateObj.setRegisterData = function(strJson) {
        var regData = JSON.parse(strJson);
        var ok = true;
        // var regBlocks = templateObj.$.ti_widget_register_page
        //   ._registerView.regBlocks;
        var regPage = document.querySelector("ti-widget-register-page");
        var regBlocks = regPage.regBlocks;
        for (var b = 0; b < regBlocks.length; b++) {
          var registersData = [];
          for (var r = 0; r < regBlocks[b].registers.length; r++) {
            var reg = regBlocks[b].registers[r];
            var regName = reg.name.toLowerCase();
            regName = regName.replace(/\&/g, "_");
            item = regData.data[b][r];
            if (item.id === regName) {
              reg.value = item.value;
              if (!templateObj.$.ti_widget_register_page.isDeferredWrite) {
                reg.valueLastWritten = reg.value;
                var intValue = gc.utils.getValueFromHexString("" + reg.value);
                templateObj.$.ti_widget_register_page.writeValueToTargetRegister(
                  intValue,
                  null,
                  b,
                  r
                );
              } else {
                templateObj.$.ti_widget_register_page._registerView.refreshRegValue(
                  b,
                  r
                );
              }
            } else {
              var msg =
                "load registers failed: register name mismatch (" +
                item.id +
                " does not match " +
                regName +
                ")";
              templateObj.logMsgAndNotify("warning", msg, false, true);
            }
          }
        }
        // if (templateObj.$.ti_widget_register_page.isDeferredWrite) {
        //   templateObj.$.ti_widget_register_page._registerView
        //     ._deferredAccessCtr++; // update the page
        // }
        if (regPage.isDeferredWrite) {
          regPage.getRegisterView()._deferredAccessCtr++;
        }
      };

      mainComponentObj.showToastMsg = function(
        msgType,
        msg,
        durationInSeconds
      ) {
        mainComponentObj.$.ti_widget_toast.hideToast();
        mainComponentObj.$.ti_widget_toast.message = msg;
        switch (msgType.toLowerCase()) {
          case "error":
            mainComponentObj.$.ti_widget_toast.iconName = "error";
            mainComponentObj.$.ti_widget_toast.backgroundColor = "red";
            mainComponentObj.$.ti_widget_toast.fontColor = "white";
            break;
          case "warning":
            mainComponentObj.$.ti_widget_toast.iconName = "warning";
            mainComponentObj.$.ti_widget_toast.backgroundColor = "orange";
            mainComponentObj.$.ti_widget_toast.fontColor = "white";
            break;
          default:
            mainComponentObj.$.ti_widget_toast.iconName = "check";
            mainComponentObj.$.ti_widget_toast.backgroundColor = "#D9F1F4";
            mainComponentObj.$.ti_widget_toast.fontColor = "#189";
            break;
        }
        mainComponentObj.$.ti_widget_toast.duration = +durationInSeconds;
        mainComponentObj.$.ti_widget_toast.showToast();
        console.log("toast msg [" + msgType + "]: " + msg);
      };

      templateObj.logMsgAndNotify = function(
        msgType,
        msg,
        showInStatusBar,
        showInToast
      ) {
        var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(
          msgType
        );
        if (showInStatusBar) {
          var statusBar = gc.services["ti-widget-statusbar"];
          if (statusBar) {
            statusBar.statusString3 = msg;
            statusBar.setIcon(iconName);
          }
        }
        if (showInToast) {
          mainComponentObj.showToastMsg(msgType, msg, 4);
        }
        templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
      };
      /* Demo of how to select a board by clicking on a custom button (e.g. with ti_widget_multidevice_app_mgr hidden):

            templateObj.selectBoard = function(event){
                var targetId = event.currentTarget.id;
                var button = document.querySelector('#'+targetId);
                templateObj.$.ti_widget_menubar.productName = button.label;
                var lcBoardName = button.label.toLowerCase().trim();
                templateObj.$.ti_widget_multidevice_app_mgr.selectedBoard = lcBoardName;
            };
            templateObj.$.ti_widget_button_board1.addEventListener('click',templateObj.selectBoard);
            templateObj.$.ti_widget_button_board2.addEventListener('click',templateObj.selectBoard);
            */
      // templateObj.$.ti_widget_label_learnmore.addEventListener(
      //   "click",
      //   function (event) {
      //     templateObj.$.ti_widget_label_productname.fontColor =
      //       "#444";
      //     templateObj.$.ti_widget_label_productdesc.fontColor =
      //       "#444";
      //     templateObj.$.ti_widget_label_learnmore.setAttribute(
      //       'hidden', true);
      //     templateObj.$.ti_widget_label_showless.removeAttribute(
      //       'hidden');
      //     templateObj.$.ti_widget_vtab_home.style.backgroundColor =
      //       "white";
      //     templateObj.$.ti_widget_tabcontainer_home.selectedIndex =
      //       1;
      //   });
      // templateObj.$.ti_widget_label_showless.addEventListener("click",
      //   function (event) {
      //     templateObj.$.ti_widget_label_productname.fontColor =
      //       "white";
      //     templateObj.$.ti_widget_label_productdesc.fontColor =
      //       "white";
      //     templateObj.$.ti_widget_label_learnmore.removeAttribute(
      //       'hidden');
      //     templateObj.$.ti_widget_label_showless.setAttribute(
      //       'hidden', true);
      //     templateObj.$.ti_widget_vtab_home.style.backgroundColor =
      //       "rgb(20, 140, 156)";
      //     templateObj.$.ti_widget_tabcontainer_home.selectedIndex =
      //       0;
      //   });

      /* Demo of how to open user guide documents in response to the user pressing an icon button:

            templateObj.openLinkOnClick = function(urlToOpen, event){
                window.open(urlToOpen);
            };
            templateObj.$.ti_widget_icon_button_userguide1.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_GUI_User_Guide.pdf'));
            templateObj.$.ti_widget_icon_button_userguide2.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_Sensored_User_Guide.pdf'));
            templateObj.$.ti_widget_icon_button_userguide3.addEventListener('click',templateObj.openLinkOnClick.bind(templateObj,'app/assets/collaterals/DRV8304x_EVM_Sensorless_User_Guide.pdf'));
                */

      // bc: TODO: if your app needs to use the ti_widget_multidevice_app_mgr,
      // rework the references to ti_widget_menubar to access the one hosted by the ti-tpslp-main
      // component
      if (templateObj.$.ti_widget_multidevice_app_mgr) {
        originalProductName = templateObj.$.ti_widget_menubar.productName;
        templateObj.$.ti_widget_vtabcontainer.addEventListener(
          "app_reset",
          function(event) {
            if (gc.connectionManager) {
              if (gc.databind) {
                gc.databind.registry
                  .getModel("my_model")
                  .getBinding("$sendPacketInterval")
                  .setValue(100);
              }
              gc.connectionManager.disconnect();
              templateObj.$.ti_widget_menubar.productName = originalProductName;
            }
            templateObj.$.ti_widget_multidevice_app_mgr.resetApp();
          }
        );
        if (templateObj.$.ti_widget_multidevice_app_mgr) {
          templateObj.$.ti_widget_multidevice_app_mgr.addEventListener(
            "device_selected",
            function(event) {
              if (gc.connectionManager) {
                // gc.connectionManager.connect();
              }
              // get the active device name from the register page
              if (event.detail && event.detail.deviceNames) {
                var deviceName = event.detail.deviceNames.toUpperCase();
                if (deviceName.length > 0) {
                  templateObj.$.ti_widget_menubar.productName = deviceName;
                }
              }
            }
          );
        }
      }
      if (templateObj.$.ti_widget_button_connect) {
        templateObj.$.ti_widget_button_connect.addEventListener(
          "click",
          function(event) {
            if (gc.connectionManager) {
              if (!templateObj) {
                templateObj = document.querySelector("#template_obj");
              }
              templateObj.$.ti_tpslp_main._connectToHardware();
            }
          }
        );
      }

      // wait for register model to be initialized
      // (NOTE: This is the example for dynamically updating the register grid view using field binding)
      // if (registerModel){
      //     var bindName = "RECOV_CNT_PFSM_INCR.INCREMENT_RECOV_CNT";
      //     var deviceJsonObj;  // keep in scope
      //     registerModel.getRegisterViewInformation(0).then(function(deviceObject) {
      //         deviceJsonObj = deviceObject;
      //         regChangedListener = new gc.databind.IChangedListener();
      //         regChangedListener.onValueChanged = _registerChangedListener.bind(this, bindName);
      //         registerBind = registerModel.getModel().getBinding(bindName);
      //         registerBind.addChangedListener(regChangedListener);
      //         deviceJsonObj.regblocks[1].attrs.isHidden = registerBind;
      //     });
      // }
    }, 1);
  });

  // gc.databind.codecsReady.then(() => {
  gc.databind.AEVMCodec.prototype.setUpdateInterval = interval => {
    let registerModel = Polymer.dom(document).querySelector(
      "ti-model-register"
    );
    registerModel
      .getModel()
      .getBinding("$refresh_interval")
      .setValue(interval);
  };
  // });

  gc.databind.registry
    .getModel("my_model")
    .getBinding("$sendPacketInterval")
    .setValue(100);
};

templateObj = document.querySelector("#template_obj");
mainComponentObj = document.querySelector("#ti_tpslp_main");
if (templateObj && mainComponentObj) {
  init();
} else {
  document.addEventListener("WebComponentsReady", init);
}
